#!/usr/bin/lua

-- Translate literal gtk types as produced by get_gtk_declarations.lua
-- to the integer code used in the hashed method signatures and to an enum
-- that C can use to extract the codes from the signature.

-- Usage:   (lua)  require "typetables"  -- defines the typecodes table
--          (shell) typetables.lua  -- prints the Ltypes enum


----------- Add types here (max 15 unless we expand bit masks) --------------

local MAXTYPE = 15

local Ltypes = {[0]="T_VOID", "T_BOOL", "T_INT", "T_UINT", "T_GUINTP",
           "T_FLOAT", "T_DOUBLE","T_STRING", "T_WIDGET", [MAXTYPE]="T_BAD"}

local paramtable = {
	["void"] = "T_VOID",
	["gboolean"] = "T_BOOL",
	["gint"] = "T_INT",
	["guint"] = "T_UINT",
	["guint *"] = "T_GUINTP",
	["gfloat"] = "T_FLOAT",
	["gdouble"] = "T_DOUBLE",
	["const char *"] = "T_STRING",
	["const gchar *"] = "T_STRING",
	["gchar *"] = "T_STRING",
	["char *"] = "T_STRING",
	["GtkWidget *"] = "T_WIDGET"
}

-- all error codes should have ERRFLAG nibble set
consts = { ERRFLAG=0x000F, ERRNARGS=0xFFFF, ERRTYPE=0xEFFF }

--------- Nothing to edit below here ----------------------------------------

local function tname_to_number(tname)
	for i, t in pairs(Ltypes) do
		if t == tname then return i end
	end
	return MAXTYPE
end

-- Print a C enum for the type codes and defines for the errors
local function print_enum()
	io.write("enum LGtypes {")
	io.write(string.format("%s=%d", Ltypes[0], 0))
	for i, t in ipairs(Ltypes) do
		io.write(string.format(", %s=%d", t, i))
	end
	io.write("};\n")
	for k,v in pairs(consts) do
		io.write(string.format("#define %s %s\n", k, v))
	end
end

-- Construct the typecodes table and some error constants for get_methods.lua
local function define_typecodes()
	typecodes = {}
	for gtk_type, Ltype in pairs(paramtable) do
		typecodes[gtk_type] = tname_to_number(Ltype)
	end
	for k,v in pairs(consts) do _G[k]=v end
end

if (...) then
	define_typecodes()    -- If imported, define the typecode table
else 
	print_enum()    -- If run freestanding, print C code
end



